// Copyright (C) 2009 - CEA - Jean-Marc Martinez
// Copyright (C) 2009-2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

//
// Compute the first order sensitivity indices, the naive way.
// Requires np+d*np^2 function evaluations, where np is the number of 
// experiments and d is the number of variables.
// Here np = 1000 and d = 2.
// This version is vectorized and faster, but requires lots of 
// memory.
//

stacksize("max");

tic();

// First variable
// Normal
mu1 = 1.5;
sigma1 = 0.5;
// Second variable
// Normal
mu2 = 3.5;
sigma2 = 2.5;
exact = nisp_productsa ( [mu1,mu2]' , [sigma1,sigma2]' );

// 
// We apply the definition of the first order sensitivity 
// indice of the first variable is 
// S1 = V(E(y|x1))/V(y)
//
// Begin with np = 10.
// Serious computation is with np >= 1000.
// More than this cannot be done in practice with this method.
np = 100;
mprintf("Number of experiments : %d\n",np+2*np^2);

//
// 1. Compute the variance of y.
// Define a sampling for x1.
u = grand(np,1,"def");
x1 = cdfnor("X",mu1*ones(np,1),sigma1*ones(np,1),u,1-u);
// Define a sampling for x2.
u = grand(np,1,"def");
x2 = cdfnor("X",mu2*ones(np,1),sigma2*ones(np,1),u,1-u);
// Merge the two samplings
x = [x1 x2];
// Perform the experiments
y = nisp_product (x);
Ey= mean(y);
Vy= variance(y);
mprintf("Expectation y = %.5f (expectation = %.5f)\n", Ey, exact.expectation);
mprintf("Variance y = %.5f (expectation = %.5f)\n", Vy, exact.var);
//
// 2. Compute E(y|x1) for several values of x1.
// Define a sampling for x1.
u = grand(np,1,"def");
x1 = cdfnor("X",mu1*ones(np,1),sigma1*ones(np,1),u,1-u);
// For each value of x1, define a sampling for x2.
u = grand(np^2,1,"def");
x2 = cdfnor("X",mu2*ones(np^2,1),sigma2*ones(np^2,1),u,1-u);
// Merge the samplings of x1 and x2.
// Replicate the sampling of x1, so that 
// 1 sample of x1 correspond to np samples of x2.
x = [x1.*.ones(np,1),x2];
// Perform the experiments
y = nisp_product (x);
// Store E(y|x1) in Ey_all(i).
// Each block of np Y-values corresponds to one sample of x2.
// Reshape y, so that each column correspond to one sample of x2.
y= matrix(y,np,np);
Ey_all = mean(y,"r")';
//
// Compute S1
S1 = variance(Ey_all)/Vy;
mprintf("S1 = %.5f (expectation = %.5f)\n", S1, exact.S1);
re = abs(S1- exact.S1)/exact.S1;
mprintf("  Relative Error = %f\n", re);
//
// 3. Compute E(y|x2) for several values of x2.
// Define a sampling for x2.
u = grand(np,1,"def");
x2 = cdfnor("X",mu2*ones(np,1),sigma2*ones(np,1),u,1-u);
// For each value of x2, define a sampling for x1.
u = grand(np^2,1,"def");
x1 = cdfnor("X",mu1*ones(np^2,1),sigma1*ones(np^2,1),u,1-u);
// Merge the samplings of x1 and x2
// Replicate the sampling of x1, so that 
// 1 sample of x2 correspond to np samples of x1.
x = [x1,x2.*.ones(np,1)];
// Perform the experiments
y = nisp_product (x);
// Store E(y|x2) in Ey_all(i)
// Each block of np Y-values corresponds to one sample of x1.
// Reshape y, so that each column correspond to one sample of x1.
y= matrix(y,np,np);
Ey_all = mean(y,"r")';
//
// Compute S1
S2 = variance(Ey_all)/Vy;
mprintf("S2 = %.5f (expectation = %.5f)\n", S2, exact.S2);
re = abs(S2- exact.S2)/exact.S2;
mprintf("  Relative Error = %f\n", re);

S12 = 1 - S1 - S2;
mprintf("S12 = %.5f (expectation = %.5f)\n", S12, exact.S12);


t = toc();
mprintf("Time = %f (s)\n", t);
//
// Load this script into the editor
//
filename = 'product_naive2.sce';
dname = get_absolute_file_path(filename);
editor ( dname + filename );

