// Copyright (C) 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html


function varargout = nisp_bruteforcesa ( varargin )
    // Compute sensitivity indices by brute force.
    //
    // Calling Sequence
    //   s = nisp_bruteforcesa ( func , nx )
    //   s = nisp_bruteforcesa ( func , nx , randgen )
    //   s = nisp_bruteforcesa ( func , nx , randgen , n )
    //   [ s , nbevalf ] = nisp_bruteforcesa ( ... )
    //
    // Parameters
    //   func : a function or a list, the name of the function to be evaluated. 
    //   nx : a 1-by-1 matrix of floating point integers, the number of inputs of the function.
    //   randgen : a function or a list, the random number generator. (default = uniform random variables)
    //   n : a 1-by-1 matrix of floating point integers (default n=10000), the number of Monte-Carlo experiments, for each sensitivity index
    //   s : a nx-by-1 matrix of doubles, the first order sensitivity indices
    //   nbevalf : a nx-by-1 matrix of doubles, the actual number of function evaluations.
    //
    // Description
    // The algorithm uses the Sobol method to compute the first order 
    // sensitivity indices.
    //
    // This function does not make any hypothesis about the dependency 
    // structure of the input random variables, i.e. it can 
    // also work if the input random variables are dependent.
    //
    // Any optional input argument equal to the empty matrix will be set to its 
    // default value.
    //
    // The function should have header y = func ( x ) where x is a m-by-nx matrix of doubles, 
    // where m is the number of experiments to perform, nx is the number of input 
    // random variables, and y is a m-by-1 matrix of doubles.
    //
    // It might happen that the function requires additionnal 
    // arguments to be evaluated.
    // In this case, we can use the following feature.
    // The function func can also be a list, with header y = f ( x , a1 , a2 , ... ).
    // In this case the func variable should hold the list (f,a1,a2,...) and the 
    // input arguments a1, a2, will be automatically be appended at the 
    // end of the calling sequence of rg.
    //
    // The random number generator must have header x = randgen ( nx , n , step , i ) where 
    //   <itemizedlist>
    // <listitem>nx is a 1-by-1 matrix of floating point integers, the number of input random variables</listitem>
    // <listitem>n is a 1-by-1 matrix of floating point integers, the number of basic Monte-Carlo experiments</listitem>
    // <listitem>step is a 1-by-1 matrix of floating point integers, the step during the 
    // algorithm. We have either step=1, or step=2. </listitem>
    // <listitem>i is a 1-by-1 matrix of floating point integers representing the index of the
    // input variable, where i is in the set {1,2,...,nx}.
    // If step=1, the value of i is [].
    // The variable i is set only if step=2.
    // </listitem>
    // <listitem>x is a n-by-nx matrix of doubles. </listitem>
    //   </itemizedlist>
    // On output, x(:,j) must contain random numbers sampled from the distribution 
    // function associated with the input variable #j, where j is in the set {1,2,...,nx}.
    //
    // There are two phases in the algorithm.
    // During step=1, a basic Monte-Carlo sampling must be generated.
    // During step=2, the values associated with variable #i must be replicated,
    // so that there are only sqrt(n) different values in x(:,i).
    // This allows to compute E(Y|Xi), with sqrt(n) different values of 
    // Xi.
    //
    // In the case where all the input variables are independent, it is 
    // straightforward to generate independent samples.
    // In the case where the input variables are dependent, the 
    // creation of the sampling must be done according to the 
    // dependency structure.
    //
    // It might happen that the random number generator requires additionnal 
    // arguments to be evaluated.
    // In this case, we can use the following feature.
    // The function randgen can also be a list, with header x = rg ( n , step , i , a1 , a2 , ... ).
    // In this case the randgen variable should hold the list (rg,a1,a2,...) and the 
    // input arguments a1, a2, will be automatically be appended at the 
    // end of the calling sequence of rg.
    // 
    // TODO : test it on a 4 input variables test case.
    //
    // TODO : compute the total sensitivity indices.
    //
    // Examples
    // // Compute the first order sensitivity indices of the ishigami function.
    // // Three random variables uniform in [-pi,pi].
    // function y=ishigami(x)
    //     a=7.
    //     b=0.1
    //     s1=sin(x(:,1))
    //     s2=sin(x(:,2))
    //     x34 = x(:,3).^4
    //     y(:,1) = s1 + a.*s2.^2 + b.*x34.*s1
    // endfunction
    // function x = myrandgen ( nx, n, step, i)
    //     if ( step == 1 ) then
    //         x = grand(n,nx,"unf",-%pi,%pi)
    //     else
    //         sn = sqrt(n)
    //         xi = grand(sn,1,"unf",-%pi,%pi)
    //         xr = grand(n,nx-1,"unf",-%pi,%pi)
    //         x = [xr(:,1:i-1),xi.*.ones(sn,1),xr(:,i:nx-1)]
    //     end
    // endfunction
    // a=7.;
    // b=0.1;
    // exact = nisp_ishigamisa(a,b);
    // n = 500;
    // nx = 3;
    // // See the sampling in step 1
    // step = 1;
    // x = myrandgen ( nx, 4, step, [])
    // // See the sampling in step 2.
    // // See how the variable #3 is replicated.
    // step = 2;
    // x = myrandgen ( nx, 16, step, 3)
    // [ s , nbevalf ] = nisp_bruteforcesa ( ishigami , nx , myrandgen , n );
    // mprintf("S1 : %f (expected = %f)\n", s(1), exact.S1);
    // mprintf("S2 : %f (expected = %f)\n", s(2), exact.S2);
    // mprintf("S3 : %f (expected = %f)\n", s(3), exact.S3);
    //
    // Authors
    //   Michael Baudin - 2011 - DIGITEO


    function x = getsampling ( nx , n , step , i , __brute_randgen__ )
        if ( typeof( __brute_randgen__ ) == "list" ) then
            // List
            __brute_randgen__f = __brute_randgen__ (1);
        end
        if ( typeof( __brute_randgen__ ) == "list" ) then
            // List
            x = __brute_randgen__f ( nx , n , step , i  , __brute_randgen__ (2:$));
        else
            // Macro or compiled macro
            x = __brute_randgen__ ( nx , n , step , i );
        end

        if ( size(x)<>[n nx] ) then
            errmsg = msprintf(gettext("%s: The size of the output xi of randgen is [%d,%d], which is different from [%d,%d]."), "nisp_bruteforcesa" , size(x,"r") , size(x,"c") , n , nx );
            error(errmsg)
        end
        // Check that, during step=2, only sqrt(n) different values 
        // are present in x(:,i) can be checked
        // to avoid wrongly using the function.
        if ( step == 2 ) then
            nxi = length(unique(x(:,i)))
            if ( nxi<>sqrt(n) ) then
                errmsg = msprintf(gettext("%s: The number of unique values in x(:,%d) out of of randgen is %d, which is different from %d."), "nisp_bruteforcesa" , i, nbxi, n );
                error(errmsg)
            end
        end

    endfunction

    function x = __nisp_sobol_rgdef__ ( nx , n , step , i )
        // x = __nisp_sobol_rgdef__ ( 3 , 16 , 1 , [] ) // Step 1
        // x = __nisp_sobol_rgdef__ ( 3 , 16 , 2 , 3 ) // Step 2, replicate i=3.
        if ( step == 1 ) then
            x = grand(n,nx,"def")
        else
            sn = sqrt(n)
            xi = grand(sn,1,"def")
            xr = grand(n,nx-1,"def")
            x = [xr(:,1:i-1),xi.*.ones(sn,1),xr(:,i:nx-1)]
        end
    endfunction

    function y = getfvalue ( x , __brute_func__ )
        if ( typeof( __brute_func__ ) == "list" ) then
            // List
            __brute_func__f = __brute_func__ (1);
            y = __brute_func__f ( x , __brute_func__ (2:$));
        else
            // Macro or compiled macro
            y = __brute_func__ ( x );
        end
    endfunction

    [lhs, rhs] = argn();
    apifun_checkrhs ( "nisp_bruteforcesa" , rhs , 2 : 4 )
    apifun_checklhs ( "nisp_bruteforcesa" , lhs , [0 1 2] )
    //
    __brute_func__ = varargin(1)
    nx = varargin(2)
    __brute_randgen__ = apifun_argindefault ( varargin , 3 , __nisp_sobol_rgdef__ )
    n = apifun_argindefault ( varargin , 4 , 10000 )
    //
    // Check input arguments
    //
    // Check type
    apifun_checktype ( "nisp_bruteforcesa" , __brute_func__ ,   "func" ,  1 , [ "function" "list" ] )
    apifun_checktype ( "nisp_bruteforcesa" , nx ,      "nx" ,     2 , "constant" )
    apifun_checktype ( "nisp_bruteforcesa" , __brute_randgen__ ,  "randgen" , 3 , [ "function" "list" ] )
    apifun_checktype ( "nisp_bruteforcesa" , n ,  "n" , 4 , "constant" )
    //
    // Check size
    apifun_checkscalar ( "nisp_bruteforcesa" , nx ,       "nx" ,       2 )
    apifun_checkscalar ( "nisp_bruteforcesa" , n ,   "n" ,   4 )
    //
    // Check content
    apifun_checkgreq ( "nisp_bruteforcesa" , nx ,  "nx" ,  2 , 1 )
    apifun_checkgreq ( "nisp_bruteforcesa" , n ,   "n" ,   4 , 1 )
    //
    // Proceed...
    // 1. Compute the variance of y.
    // Define a sampling for x
    step = 1;
    x = getsampling ( nx , n , step , [] , __brute_randgen__ );
    // Perform the experiments
    y = getfvalue ( x , __brute_func__ );
    Ey= mean(y);
    Vy= variance(y);
    //
    // 2. Compute E(y|xi) for several values of xi.
    step = 2
    for i = 1 : nx
        // Get the sampling for xi.
        // Replicate the sampling of x1, so that
        // 1 sample of x1 correspond to np samples of x2.
        x = getsampling ( nx , n^2 , step , i , __brute_randgen__ );
        // Perform the experiments
        y = getfvalue ( x , __brute_func__ );
        // Store E(y|xi) in Ey_all(k), for k=1,2,...,n.
        // Each block of n Y-values corresponds to one sample of xi.
        // Reshape y, so that each column correspond to one sample of xi.
        y= matrix(y,n,n);
        Ey_all = mean(y,"r")';
        //
        // Compute Si
        s(i) = variance(Ey_all)/Vy;
    end
    nbevalf = n + nx*n^2

    //
    // Set outputs
    varargout(1) = s
    varargout(2) = nbevalf
endfunction

