// Copyright (C) 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html


function p = nisp_expcdf ( varargin )
  // Computes the Exponential CDF.
  //
  // Calling Sequence
  //   p = nisp_expcdf ( x )
  //   p = nisp_expcdf ( x , lambda )
  //
  // Parameters
  // x: a matrix of doubles
  // lambda : a matrix of doubles
  // p: a matrix of doubles, the probability
  //
  // Description
  //   This function computes the Exponential CDF.
  //
  //   Any scalar input argument is expanded to a matrix of doubles of the same size 
  //   as the other input arguments.
  //
  // Any optional input argument equal to the empty matrix will be set to its
  // default value.
  // 
  // TODO : improve implementation (check inf, nan, etc...)
  //
  // Examples
  // // http://en.wikipedia.org/wiki/Exponential_distribution
  // scf();
  // x = linspace(0,5,1000);
  // p = nisp_expcdf ( x , 0.5 );
  // plot(x,p, "r-" );
  // p = nisp_expcdf ( x , 1 );
  // plot(x,p, "m-" );
  // p = nisp_expcdf ( x , 1.5 );
  // plot(x,p, "c-" );
  // xtitle("Exponential Cumulated Distribution Function","X","P(X)");
  // legend(["lambda=0.5","lambda=","lambda=1.5"]);
  //
  // Authors
  // Copyright (C) 2008-2011 - INRIA - Michael Baudin
  //
  // Bibliography
  // Wikipedia, Exponential distribution function, http://en.wikipedia.org/wiki/Exponential_distribution

    [lhs, rhs] = argn()
    apifun_checkrhs ( "nisp_expcdf" , rhs , 1:2 )
    apifun_checklhs ( "nisp_expcdf" , lhs , 0:1 )
    //
    x = varargin(1)
    lambda = apifun_argindefault ( varargin , 2 , 1 )
    //
    // Check type
    apifun_checktype ( "nisp_expcdf" , x , "x" , 1 , "constant" )
    apifun_checktype ( "nisp_expcdf" , lambda , "lambda" , 2 , "constant" )
    //
    // Check size : OK
    //
    // Check content
    tiny = number_properties("tiny")
    apifun_checkgreq ( "nisp_expcdf" , lambda , "lambda" , 2 , tiny )
    //
    // Proceed ...
    [ x , lambda ] = nisp_expandvar ( x , lambda )
    //
  q = exp(-lambda.*x)
  p = 1-q
endfunction

