// Copyright (C) 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html


function x = nisp_expinv ( varargin )
  // Computes the Exponential quantile.
  //
  // Calling Sequence
  //   x = nisp_expinv ( p )
  //   x = nisp_expinv ( p , lambda )
  //
  // Parameters
  // p: a matrix of doubles, the probability
  // lambda : a matrix of doubles
  // x: a matrix of doubles
  //
  // Description
  //   This function computes the Exponential Quantile.
  //
  //   Any scalar input argument is expanded to a matrix of doubles of the same size 
  //   as the other input arguments.
  //
  // Any optional input argument equal to the empty matrix will be set to its
  // default value.
  //
  // This function might be inaccurate when p is close to 1. 
  // 
  // TODO : improve implementation (check inf, nan, etc...)
  //
  // Examples
  // // Check the inversion
  // lambda=2;
  // x=3;
  // p = nisp_expcdf ( x , lambda ); // p = 0.9975212
  // x = nisp_expinv ( p , lambda ) // must be 3
  //
  // Authors
  // Copyright (C) 2008-2011 - INRIA - Michael Baudin
  //
  // Bibliography
  // Wikipedia, Exponential distribution function, http://en.wikipedia.org/wiki/Exponential_distribution

    [lhs, rhs] = argn()
    apifun_checkrhs ( "nisp_expinv" , rhs , 1:2 )
    apifun_checklhs ( "nisp_expinv" , lhs , 0:1 )
    //
    p = varargin(1)
    lambda = apifun_argindefault ( varargin , 2 , 1 )
    //
    // Check type
    apifun_checktype ( "nisp_expinv" , p , "p" , 1 , "constant" )
    apifun_checktype ( "nisp_expinv" , lambda , "lambda" , 2 , "constant" )
    //
    // Check size : OK
    //
    // Check content
    apifun_checkrange ( "nisp_expinv" , p , "p" , 1 , 0 , 1 )
    tiny = number_properties("tiny")
    apifun_checkgreq ( "nisp_expinv" , lambda , "lambda" , 2 , tiny )
    //
    // Proceed ...
    [ p , lambda ] = nisp_expandvar ( p , lambda )
    //

  x = - log(1-p)./lambda
endfunction

