// Copyright (C) 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

function p = nisp_lognormalcdf ( varargin )
  // Computes the Lognormal CDF.
  //
  // Calling Sequence
  //   p = nisp_lognormalcdf ( x )
  //   p = nisp_lognormalcdf ( x , mu )
  //   p = nisp_lognormalcdf ( x , mu , sigma )
  //
  // Parameters
  // x: a matrix of doubles
  // mu: a matrix of doubles, the mean of the underlying normal variable (default mu = 0).
  // sigma: a matrix of doubles, the variance of the underlying normal variable (default sigma = 1).
  // p: a matrix of doubles, the probability
  //
  // Description
  //   This function computes the Lognormal Cumulated Density Function.
  //
  //   Any scalar input argument is expanded to a matrix of doubles of the same size 
  //   as the other input arguments.
  //
  // Any optional input argument equal to the empty matrix will be set to its
  // default value.
  //
  // TODO : improve implementation (check inf, nan, etc...)
  //
  // Examples
  // // See http://en.wikipedia.org/wiki/File:Lognormal_distribution_CDF.png
  // scf();
  // x = linspace ( 0 , 3 , 1000 );
  // p = nisp_lognormalcdf ( x , 0.0 , 10 );
  // plot ( x , p , "k" );
  // p = nisp_lognormalcdf ( x , 0.0 , 3/2 );
  // plot ( x , p , "b" );
  // p = nisp_lognormalcdf ( x , 0.0 , 1 );
  // plot ( x , p , "g" );
  // p = nisp_lognormalcdf ( x , 0.0 , 1/2 );
  // plot ( x , p , "y" );
  // p = nisp_lognormalcdf ( x , 0.0 , 1/4 );
  // plot ( x , p , "r" );
  // legend ( ["s=10" "s=3/2" "s=1" "s=1/2" "s=1/4"] );
  // xtitle("The log-normale cumulated distribution function","X","P(x)");
  //
  // Authors
  // Copyright (C) 2008-2011 - INRIA - Michael Baudin
  //
  // Bibliography
  // Dider Pelat, "Bases et méthodes pour le traitement de données", section 8.2.8, "Loi log-normale".
  // Wikipedia, Lognormal probability distribution function, http://en.wikipedia.org/wiki/File:Lognormal_distribution_PDF.png
  // Wikipedia, Lognormal cumulated distribution function, http://en.wikipedia.org/wiki/File:Lognormal_distribution_CDF.png

    [lhs, rhs] = argn()
    apifun_checkrhs ( "nisp_lognormalcdf" , rhs , 1:3 )
    apifun_checklhs ( "nisp_lognormalcdf" , lhs , 0:1 )
    //
    x = varargin(1)
    mu = apifun_argindefault ( varargin , 2 , 0 )
    sigma = apifun_argindefault ( varargin , 3 , 1 )
    //
    // Check type
    apifun_checktype ( "nisp_lognormalcdf" , x , "x" , 1 , "constant" )
    apifun_checktype ( "nisp_lognormalcdf" , mu , "mu" , 2 , "constant" )
    apifun_checktype ( "nisp_lognormalcdf" , sigma , "sigma" , 3 , "constant" )
    //
    // Check size : OK
    //
    // Check content
    tiny = number_properties("tiny")
    apifun_checkgreq ( "nisp_lognormalcdf" , sigma , "sigma" , 3 , tiny )
    //
    // Proceed ...
    [ x , mu , sigma ] = nisp_expandvar ( x , mu , sigma )
    //
  ii = find ( x<=0 )
  p(ii) = 0
  ii = find ( x>0 )
  c = - (log(x(ii)) - mu(ii))./sigma(ii)/sqrt(2)
  p(ii) = 0.5 * erfc ( c(ii) )
endfunction

