// Copyright (C) 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html



function x = nisp_lognormalinv ( varargin )
  // Computes the Lognormal quantile.
  //
  // Calling Sequence
  //   x = nisp_lognormalinv ( p )
  //   x = nisp_lognormalinv ( p , mu )
  //   x = nisp_lognormalinv ( p , mu , sigma )
  //
  // Parameters
  // p: a matrix of doubles, the probability. Must be in the range [0,1].
  // mu: a matrix of doubles, the mean of the underlying normal variable (default mu = 0).
  // sigma: a matrix of doubles, the variance of the underlying normal variable (default sigma = 1).
  // x: a matrix of doubles
  //
  // Description
  //   This function computes the Lognormal quantile.
  //
  //   Any scalar input argument is expanded to a matrix of doubles of the same size 
  //   as the other input arguments.
  //
  // Any optional input argument equal to the empty matrix will be set to its
  // default value.
  //
  // TODO : improve implementation (check inf, nan, etc...)
  //
  // Examples
  // //
  // // Check the inverse lognormale
  // mu = 1;
  // sigma=10;
  // x=2;
  // p = nisp_lognormalcdf ( x , mu , sigma ); // 0.4877603
  // x = nisp_lognormalinv ( p , mu , sigma ) // Must be 2
  //
  // Authors
  // Copyright (C) 2008-2011 - INRIA - Michael Baudin
  //
  // Bibliography
  // Dider Pelat, "Bases et méthodes pour le traitement de données", section 8.2.8, "Loi log-normale".
  // Wikipedia, Lognormal probability distribution function, http://en.wikipedia.org/wiki/File:Lognormal_distribution_PDF.png
  // Wikipedia, Lognormal cumulated distribution function, http://en.wikipedia.org/wiki/File:Lognormal_distribution_CDF.png

    [lhs, rhs] = argn()
    apifun_checkrhs ( "nisp_lognormalinv" , rhs , 1:3 )
    apifun_checklhs ( "nisp_lognormalinv" , lhs , 0:1 )
    //
    p = varargin(1)
    mu = apifun_argindefault ( varargin , 2 , 0 )
    sigma = apifun_argindefault ( varargin , 3 , 1 )
    //
    // Check type
    apifun_checktype ( "nisp_lognormalinv" , p , "p" , 1 , "constant" )
    apifun_checktype ( "nisp_lognormalinv" , mu , "mu" , 2 , "constant" )
    apifun_checktype ( "nisp_lognormalinv" , sigma , "sigma" , 3 , "constant" )
    //
    // Check size : OK
    //
    // Check content
    apifun_checkrange ( "nisp_lognormal" , p , "p" , 1 , 0 , 1 )
    tiny = number_properties("tiny")
    apifun_checkgreq ( "nisp_lognormal" , sigma , "sigma" , 3 , tiny )
    //
    // Proceed ...
    [ p , mu , sigma ] = nisp_expandvar ( p , mu , sigma )
    //
  c = nisp_erfcinv(2*p)
  x = exp(mu - c.*sqrt(2).*sigma)
endfunction

