// Copyright (C) 2011 - INRIA - Michael Baudin
// Copyright (C) 2011 - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html


function p = nisp_lognormalpdf ( varargin )
  // Computes the Lognormal PDF.
  //
  // Calling Sequence
  //   p = nisp_lognormalpdf ( x )
  //   p = nisp_lognormalpdf ( x , mu )
  //   p = nisp_lognormalpdf ( x , mu , sigma )
  //
  // Parameters
  // x: a matrix of doubles
  // mu: a matrix of doubles, the mean of the underlying normal variable (default mu = 0).
  // sigma: a matrix of doubles, the variance of the underlying normal variable (default sigma = 1).
  // p: a matrix of doubles, the probability
  //
  // Description
  //   This function computes the Lognormal PDF.
  //
  //   Any scalar input argument is expanded to a matrix of doubles of the same size 
  //   as the other input arguments.
  //
  // Any optional input argument equal to the empty matrix will be set to its
  // default value.
  //
  // The Lognormal distribution with parameters mu and sigma has density
  //
  // <latex>
  // \begin{eqnarray}
  // f(x) = \frac{1}{\sigma x \sqrt{2 \pi}} \exp\left(-\frac{1}{2} \frac{(\ln(x) - \mu)^2}{\sigma^2}\right), 
  // \end{eqnarray}
  // </latex>
  //
  // for x > 0, and f(x)=0 if not.
  //
  // TODO : improve implementation (check inf, nan, etc...)
  //
  // Examples
  // // See Dider Pelat, "Bases et méthodes pour le traitement de données"
  // scf();
  // x = linspace ( 0 , 5 , 1000 );
  // p = nisp_lognormalpdf ( x , 0.0 , 1.0 );
  // plot ( x , p );
  // xtitle("The log-normale probability distribution function","X","P(x)");
  //
  // // See http://en.wikipedia.org/wiki/File:Lognormal_distribution_PDF.png
  // scf();
  // x = linspace ( 0 , 5 , 1000 );
  // p = nisp_lognormalpdf ( x , 0.0 , 10 );
  // plot ( x , p , "k" );
  // p = nisp_lognormalpdf ( x , 0.0 , 3/2 );
  // plot ( x , p , "b" );
  // p = nisp_lognormalpdf ( x , 0.0 , 1 );
  // plot ( x , p , "g" );
  // p = nisp_lognormalpdf ( x , 0.0 , 1/2 );
  // plot ( x , p , "y" );
  // p = nisp_lognormalpdf ( x , 0.0 , 1/4 );
  // plot ( x , p , "r" );
  // legend ( ["s=10" "s=3/2" "s=1" "s=1/2" "s=1/4"] );
  // xtitle("The log-normale probability distribution function","X","P(x)");
  //
  // Authors
  // Copyright (C) 2008-2011 - INRIA - Michael Baudin
  //
  // Bibliography
  // Dider Pelat, "Bases et méthodes pour le traitement de données", section 8.2.8, "Loi log-normale".
  // Wikipedia, Lognormal probability distribution function, http://en.wikipedia.org/wiki/File:Lognormal_distribution_PDF.png
  // Wikipedia, Lognormal cumulated distribution function, http://en.wikipedia.org/wiki/File:Lognormal_distribution_CDF.png

    [lhs, rhs] = argn()
    apifun_checkrhs ( "nisp_lognormalpdf" , rhs , 1:3 )
    apifun_checklhs ( "nisp_lognormalpdf" , lhs , 0:1 )
    //
    x = varargin(1)
    mu = apifun_argindefault ( varargin , 2 , 0 )
    sigma = apifun_argindefault ( varargin , 3 , 1 )
    //
    // Check type
    apifun_checktype ( "nisp_lognormalpdf" , x , "x" , 1 , "constant" )
    apifun_checktype ( "nisp_lognormalpdf" , mu , "mu" , 2 , "constant" )
    apifun_checktype ( "nisp_lognormalpdf" , sigma , "sigma" , 3 , "constant" )
    //
    // Check size : OK
    //
    // Check content
    tiny = number_properties("tiny")
    apifun_checkgreq ( "nisp_lognormalpdf" , sigma , "sigma" , 3 , tiny )
    //
    // Proceed ...
    [ x , mu , sigma ] = nisp_expandvar ( x , mu , sigma )
    //
  ii = find ( x<=0 )
  p(ii) = 0
  ii = find ( x>0 )
  c = (log(x(ii)) - mu(ii)) ./sigma(ii)
  p(ii) = exp ( -0.5 .* c.^2 ) ./ ( sigma(ii) .* x(ii) * sqrt(2*%pi) )
endfunction

