// Copyright (C) 2009 - CEA - Jean-Marc Martinez
// Copyright (C) 2010 - 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html



function exact = nisp_productsa ( mu , sigma )
  // Exact sensitivity analysis for the Product function
  //
  // Calling Sequence
  //   exact = nisp_productsa ( mu , sigma )
  //
  // Parameters
  // mu : a 2-by-1 matrix of doubles, the means of X1 and X2
  // sigma : a 1-by-1 matrix of doubles, the standard deviation of X1 a,d X2
  // exact: a struct with the following fields: expectation, var, S1, S2, S12, ST1, ST2.
  //
  // Description
  //   This function returns exact sensitivity indices of the Product function.
  // We consider two random variables:
  // * a normal random variable with mean mu1 and standard deviation sigma1.
  // * a normal random variable with mean mu2 and standard deviation sigma2.
  //
  // Examples
  // mu1 = 1.5;
  // sigma1 = 0.5;
  // mu2 = 3.5;
  // sigma2 = 2.5;
  // exact = nisp_productsa ( [mu1,mu2]' , [sigma1,sigma2]' )
  //
  // Authors
  // Copyright (C) 2008-2011 - INRIA - Michael Baudin
  // Copyright (C) 2009 - CEA - Jean-Marc Martinez
  //
  // Bibliography
  // "Sensitivity analysis in practice", Saltelli, Tarantolla, Compolongo, Ratto, Wiley, 2004
  // "An importance quantification technique in uncertainty analysis for computer models", Ishigami, Homma, 1990, Proceedings of the ISUMA'90. First international symposium on uncertainty modelling and Analysis, University of Maryland, USA, pp. 398-403.

    [lhs, rhs] = argn();
    apifun_checkrhs ( "nisp_productsa" , rhs , 2 : 2 )
    apifun_checklhs ( "nisp_productsa" , lhs , 0:1 )
    //
    //
    // Check input arguments
    //
    // Check type
    apifun_checktype ( "nisp_productsa" , mu ,   "mu" ,  1 , "constant" )
    apifun_checktype ( "nisp_productsa" , sigma ,   "sigma" ,  2 , "constant" )
    //
    // Check size
    apifun_checkdims ( "nisp_productsa" , mu ,    "mu" ,    1 , [2 1] )
    apifun_checkdims ( "nisp_productsa" , sigma ,    "sigma" ,    2 , [2 1] )
    //
    // Check content
    apifun_checkgreq  ( "nisp_productsa" , sigma ,    "sigma" ,    2 , 0 )
    //

    exact.expectation = mu(1)*mu(2);
    exact.var = mu(2)^2*sigma(1)^2 + mu(1)^2*sigma(2)^2 + sigma(1)^2*sigma(2)^2;
    exact.S1 = ( mu(2)^2*sigma(1)^2 ) / exact.var;
    exact.S2 = ( mu(1)^2*sigma(2)^2 ) / exact.var;
    exact.S12 = ( sigma(1)^2 * sigma(2)^2 ) / exact.var;
    exact.ST1 = exact.S1 + exact.S12;
    exact.ST2 = exact.S2 + exact.S12;
endfunction

