// Copyright (C) 2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html


function varargout = nisp_sobolsa ( varargin )
    // Compute sensitivity indices by Sobol, Ishigami, Homma.
    //
    // Calling Sequence
    //   s = nisp_sobolsa ( func , nx )
    //   s = nisp_sobolsa ( func , nx , randgen )
    //   s = nisp_sobolsa ( func , nx , randgen , n )
    //   [ s , nbevalf ] = nisp_sobolsa ( ... )
    //   [ s , nbevalf , st ] = nisp_sobolsa ( ... )
    //   [ s , nbevalf , st , mi , si ] = nisp_sobolsa ( ... )
    //
    // Parameters
    //   func : a function or a list, the name of the function to be evaluated. 
    //   nx : a 1-by-1 matrix of floating point integers, the number of inputs of the function.
    //   randgen : a function or a list, the random number generator. (default = uniform random variables)
    //   n : a 1-by-1 matrix of floating point integers (default n=10000), the number of Monte-Carlo experiments, for each sensitivity index
    //   s : a nx-by-1 matrix of doubles, the first order sensitivity indices
    //   nbevalf : a nx-by-1 matrix of doubles, the actual number of function evaluations.
    //   st : a nx-by-1 matrix of doubles, the total sensitivity indices
    //   mi : a m-by-nx matrix of doubles, the multi-indices of the variables in si, where m=2^nx - 1. Each row in mi represents a group of variables. We have mi(k,i) = 1 if Xi is in the group of variables and 0 if not. 
    //   si : a m-by-1 matrix of doubles, the partial sensitivity indices.
    //
    // Description
    // The algorithm uses the Sobol method to compute the first order 
    // sensitivity indices.
    //
    // This method assumes that all the input random variables are 
    // independent.
    //
    // Any optional input argument equal to the empty matrix will be set to its
    // default value.
    //
    // The function must have the header 
	//
	// <screen>
	// y = func ( x ) 
	// </screen>
	//
	// where x is a m-by-nx matrix of doubles, 
    // where m is the number of experiments to perform, nx is the number of input 
    // random variables, and y is a m-by-1 matrix of doubles.
    //
    // It might happen that the function requires additionnal 
    // arguments to be evaluated.
    // In this case, we can use the following feature.
    // The function func can also be the list (f,a1,a2,...), 
	// where f is the function with header:
	//
	// <screen>
	// y = f ( x , a1 , a2 , ... )
	// </screen>
	// 
    // and the input arguments a1, a2, ... are automatically appended at the 
    // end of the calling sequence of f.
    //
    // The random number generator must have the header 
	//
	// <screen>
	// x = randgen ( m , i )
	// </screen>
	//
	// where 
    //   <itemizedlist>
    // <listitem><para>m is a 1-by-1 matrix of floating point integers, the number of 
    // experiments to perform</para></listitem>
    // <listitem><para>i is a 1-by-1 matrix of floating point integers representing the index of the 
    // input variable,</para></listitem>
    // <listitem><para>x is a m-by-1 matrix of doubles. </para></listitem>
    //   </itemizedlist>
    // On output, x must contain random numbers sampled from the distribution 
    // function associated with the input variable #i, where i is in the set {1,2,...,nx}.
    //
    // Since the input random variables are independent, we can generate 
    // the samples associated with Xi independently from the samples for 
    // Xj, for i not equal to j.
    // This is why the callback randgen only needs the index i of the 
    // input random variable.
    // If there is some dependency between the inputs (e.g. correlation), 
    // then the randgen callback would require to generate all the samples 
    // dependently one from the other.
    // In this case, the current function cannot be used.
    //
    // It might happen that the random number generator requires additionnal 
    // arguments to be evaluated.
    // In this case, we can use the following feature.
    // The function randgen can also be the list (rg,a1,a2,...), 
	// where rg is a function with header :
	// 
	// <screen>
	// u = rg ( m , i , a1 , a2 , ... )
	// </screen>
	// 
    // and the input arguments a1, a2, ... are automatically appended at the 
    // end of the calling sequence of rg.
	//
	// On output, the sensitivity indices are forced to be in the range [0,1].
    //
    // Examples
    // // Compute the first order sensitivity indices of the ishigami function.
    // // Three random variables uniform in [-pi,pi].
    // function y = ishigami (x)
    //   a=7.
    //   b=0.1
    //   s1=sin(x(:,1))
    //   s2=sin(x(:,2))
    //   x34 = x(:,3).^4
    //   y(:,1) = s1 + a.*s2.^2 + b.*x34.*s1
    // endfunction
    // function x = myrandgen ( m , i )
    //   x = grand(m,1,"unf",-%pi,%pi)
    // endfunction
    // a=7.;
    // b=0.1;
    // exact = nisp_ishigamisa ( a , b )
    // n = 1000;
    // nx = 3;
    // [ s , nbevalf ] = nisp_sobolsa ( ishigami , nx , myrandgen , n );
    // mprintf("S1 : %f (expected = %f)\n", s(1), exact.S1);
    // mprintf("S2 : %f (expected = %f)\n", s(2), exact.S2);
    // mprintf("S3 : %f (expected = %f)\n", s(3), exact.S3);
    //
    // // See the variability of the sensitivity indices.
    // for k = 1 : 100
    //   [ s , nbevalf ] = nisp_sobolsa ( ishigami , nx , myrandgen , 1000 );
    //   sall(k,:) = s';
    // end
    // scf();
    // subplot(2,2,1);
    // histplot(10,sall(:,1));
    // xtitle("Variability of the sensitivity index for X1","S1","Frequency");
    // subplot(2,2,2);
    // histplot(10,sall(:,2));
    // xtitle("Variability of the sensitivity index for X2","S2","Frequency");
    // subplot(2,2,3);
    // histplot(10,sall(:,3));
    // xtitle("Variability of the sensitivity index for X3","S3","Frequency");
    //
    // // See the convergence of the sensitivity indices
    // n=10;
    // stacksize("max");
    // for k = 1 : 100
    //   tic();  
    //   [ s , nbevalf ] = nisp_sobolsa ( ishigami , nx , myrandgen , n );
    //   sc(k,:) = s';
    //   t = toc();
    //   mprintf("Run #%d, n=%d, t=%.2f (s)\n",k,n,t);
    //   if ( t > 1 ) then
    //     break
    //   end
    //   n = 1.2*n;
    // end
    // h = scf();
    // subplot(1,2,1);
    // plot(1:k,sc(1:k,1),"bx-");
    // plot(1:k,sc(1:k,2),"ro-");
    // plot(1:k,sc(1:k,3),"g*-");
    // xtitle("Convergence of the sensitivity indices","Number of simulations","S");
    // legend(["S1","S2","S3"]);
    // subplot(1,2,2);
    // plot(1:k,abs(sc(1:k,1)-exact.S1),"bx-");
    // plot(1:k,abs(sc(1:k,2)-exact.S2),"ro-");
    // plot(1:k,abs(sc(1:k,3)-exact.S3),"g*-");
    // xtitle("Convergence of the sensitivity indices","Number of simulations","|S-exact|");
    // legend(["S1","S2","S3"]);
    // h.children(1).log_flags="lnn";
    //
    // // Compute the total sensivity indices.
    // [ s , nbevalf , st ] = nisp_sobolsa ( ishigami , nx , myrandgen )
    //
    // // Compute all the sensivity indices.
    // [ s , nbevalf , st , mi , si ] = nisp_sobolsa ( ishigami , nx , myrandgen )
    // // The ANOVA decomposition can be seen more easily in
    // [mi si]
    // // The sum of si is 1.
    // sum(si) // expected = 1.
    //
    // Authors
    //   Michael Baudin - 2011 - DIGITEO

    function s = sensitivityindex(ya,yc)
        // Returns the sensitivity index associated with experiments ya and yc.
        C = nisp_cov (ya, yc)
        s= C(1,2)/ (st_deviation(ya) * st_deviation(yc))
		s = max(s,0.)
		s = min(s,1.)
    endfunction

    function A = getsampling ( n , nx , __sobolfo_randgen__ )
        if ( typeof( __sobolfo_randgen__ ) == "list" ) then
            // List
            __sobolfo_randgen__f = __sobolfo_randgen__ (1);
        end
        A = zeros(n , nx)
        for i = 1 : nx
            if ( typeof( __sobolfo_randgen__ ) == "list" ) then
                // List

                Ai = __sobolfo_randgen__f ( n , i , __sobolfo_randgen__ (2:$));
            else
                // Macro or compiled macro
                Ai = __sobolfo_randgen__ ( n , i );
            end

            if ( size(Ai)<>[n 1] ) then
                errmsg = msprintf(gettext("%s: The size of the output of randgen is [%d,%d], which is different from [%d,%d]."), "nisp_sobolsa" , size(Ai,"r") , size(Ai,"c") , n , 1 );
                error(errmsg)
            end
            A(:,i) = Ai
        end
    endfunction

    function x = __nisp_sobol_rgdef__ ( m , i )
        x = grand(m,1,"def")
    endfunction

    function y = getfvalue ( x , __sobolfo_func__ )
        if ( typeof( __sobolfo_func__ ) == "list" ) then
            // List
            __sobolfo_func__f = __sobolfo_func__ (1);
            y = __sobolfo_func__f ( x , __sobolfo_func__ (2:$));
        else
            // Macro or compiled macro
            y = __sobolfo_func__ ( x );
        end
    endfunction

    warnobsolete("nisp_sobolsaFirst", "2.5")

    [lhs, rhs] = argn();
    apifun_checkrhs ( "nisp_sobolsa" , rhs , 2 : 4 )
    apifun_checklhs ( "nisp_sobolsa" , lhs , [0 1 2 3 5] )
    //
    __sobolfo_func__ = varargin(1)
    nx = varargin(2)
    __sobolfo_randgen__ = apifun_argindefault ( varargin , 3 , __nisp_sobol_rgdef__ )
    n = apifun_argindefault ( varargin , 4 , 10000 )
    //
    // Check input arguments
    //
    // Check type
    apifun_checktype ( "nisp_sobolsa" , __sobolfo_func__ ,   "func" ,  1 , [ "function" "list" ] )
    apifun_checktype ( "nisp_sobolsa" , nx ,      "nx" ,     2 , "constant" )
    apifun_checktype ( "nisp_sobolsa" , __sobolfo_randgen__ ,  "randgen" , 3 , [ "function" "list" ] )
    apifun_checktype ( "nisp_sobolsa" , n ,  "n" , 4 , "constant" )
    //
    // Check size
    apifun_checkscalar ( "nisp_sobolsa" , nx ,       "nx" ,       2 )
    apifun_checkscalar ( "nisp_sobolsa" , n ,   "n" ,   4 )
    //
    // Check content
    apifun_checkgreq ( "nisp_sobolsa" , nx ,  "nx" ,  2 , 1 )
    apifun_checkgreq ( "nisp_sobolsa" , n ,   "n" ,   4 , 1 )
    //
    // Proceed...
    // Create a sampling A
    A = getsampling ( n , nx , __sobolfo_randgen__ )
    // Create a sampling B
    B = getsampling ( n , nx , __sobolfo_randgen__ )
    // Perform the experiments in A
    ya = getfvalue ( A , __sobolfo_func__ )
    // Compute the sensitivity indices
    for i = 1 : nx
        C(:,1:i-1) = B(:,1:i-1);
        C(:,i)=A(:,i);
        C(:,i+1:nx) = B(:,i+1:nx);
        yc = getfvalue ( C , __sobolfo_func__ )
        s(i) = sensitivityindex ( ya , yc );
    end
    nbevalf = n + nx * n
    //
    // Set outputs
    varargout(1) = s
    varargout(2) = nbevalf
    if ( lhs >= 3 ) then 
        //
        // If total sensitivity indices are to compute, compute them.
        for i = 1 : nx
            C(:,1:i-1) = A(:,1:i-1);
            C(:,i)=B(:,i);
            C(:,i+1:nx) = A(:,i+1:nx);
            yc = getfvalue ( C , __sobolfo_func__ )
            st(i) = 1 - sensitivityindex ( ya , yc );
        end
        nbevalf = nbevalf + nx * n
        varargout(2) = nbevalf
        varargout(3) = st
    end
    if ( lhs == 5 ) then 
        //
        // If partial sensitivity indices are to compute, compute them.
        //
        // Compute all the combinations of variables Xi1, Xi2, ..., Xip.
        // If nx=3, the matrix is :
        // 0 0 0
        // 1 0 0
        // 0 1 0
        // 0 0 1
        // 1 1 0
        // 1 0 1
        // 0 1 1
        // 1 1 1
        // It represents the possible combinations of variables: 
        // 1 means that Xi is in the subset, 0 means no variable Xi.
        mi = specfun_combinerepeat ( [1 0] , nx )'
        ks = sum(mi,"c")
        // Sort by increasing number of variables.
        [ignored,korder]=gsort(ks,"g","i")
        mi = mi(korder,:)
        // Eliminate the combination of variables 
        // where there are less than 1 variables, i.e. delete 
        // the row [0 0 0 ... 0].
        mi(1,:)=[]
        nrows = size(mi,"r")
        si=zeros(nrows,1)
        // The first nx sensitivity indices are already 
        // computed: just store them.
        for k = 1 : nx
            // Get the variable i in this row: 
            // there is a unique non-zero in the row k.
            i = find(mi(k,:)==1)
            si(k) = s(i)
        end
        // Compute the sensitivity indices for all these combinations
        // of at least two variables.
        for k = nx + 1 : nrows
            // Compute the sensitivity index for the group of 
            // variables in the row mi(k,:).
            knovar = find(mi(k,:)==0);
            C(:,knovar) = B(:,knovar);
            kvar = find(mi(k,:)==1)
            C(:,kvar)=A(:,kvar);
            yc = getfvalue ( C , __sobolfo_func__ );
            si(k) = sensitivityindex(ya,yc);
            // Remove the lower order sensitivity indices.
            for i = 1 : k-1
                ivar = find(mi(i,:)==1);
                // The sensitivity index si(i) must be subtracted 
                // from si(k) is all variables in ivar are in kvar.
                tf = specfun_ismember ( ivar , kvar );
                if ( and(tf) ) then
                    si(k) = si(k) - si(i);
                end
            end
        end
        nbevalf = nbevalf + ( nrows - nx -1 ) * n
        varargout(2) = nbevalf
        varargout(4) = mi
        varargout(5) = si
    end
endfunction

