// Copyright (C) 2010 - 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html



function y = nisp_sum ( x , a )
  // Returns the value of the sum function
  //
  // Calling Sequence
  //   y = nisp_sum ( x )
  //
  // Parameters
  // a: a 5-by-1 matrix of doubles
  // x: a np-by-2 matrix of doubles, where np is the number of experiments.
  // y: a np-by-1 matrix of doubles
  //
  // Description
  //   This function returns the value of the Sum function.
  //
  // This function is defined by 
  //
  // <latex>
  // \begin{eqnarray}
  // Y = a_1 X_1 + a_2 X_2 + a_3 X_3 + a_4 X_4 + a_5
  // \end{eqnarray}
  // </latex>
  //
  // where X1, X2, X3, X4 are normal random variables with means 
  // mu1,mu2,mu3,mu4 and standard variations sigma1,sigma2,sigma3,sigma4.
  //
  // Examples
  // x = [1 2 3 4];
  // a = [5 6 7 8 9]';
  // y = nisp_sum (x,a)
  // expected = 79
  //
  // Authors
  // Copyright (C) 2008-2011 - INRIA - Michael Baudin
  //
  // Bibliography
  // "Sensitivity analysis in practice", Saltelli, Tarantolla, Compolongo, Ratto, Wiley, 2004
  // "An importance quantification technique in uncertainty analysis for computer models", Ishigami, Homma, 1990, Proceedings of the ISUMA'90. First international symposium on uncertainty modelling and Analysis, University of Maryland, USA, pp. 398-403.

    [lhs, rhs] = argn();
    apifun_checkrhs ( "nisp_sum" , rhs , 2 : 2 )
    apifun_checklhs ( "nisp_sum" , lhs , 0:1 )
    //
    //
    // Check input arguments
    //
    // Check type
    apifun_checktype ( "nisp_sum" , x ,   "x" ,  1 , "constant" )
    apifun_checktype ( "nisp_sum" , a ,   "a" ,  2 , "constant" )
    //
    // Check size
    apifun_checkdims ( "nisp_sum" , x ,    "x" ,    1 , [size(x,"r") 4] )
    apifun_checkdims ( "nisp_sum" , a ,    "a" ,    2 , [5 1] )
    //
    // Check content
    // Nothing to check
    //

  y(:) = a(1)*x(:,1) + a(2)*x(:,2) + a(3)*x(:,3) + a(4)*x(:,4) + a(5)
endfunction

