// Copyright (C) 2010 - 2011 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html



function exact = nisp_sumsa ( a , mu , sigma )
  // Returns the sensitivity indices of the Sum function
  //
  // Calling Sequence
  //   exact = nisp_sumsa ( a , mu , sigma )
  //
  // Parameters
  // a: a 5-by-1 matrix of doubles
  // mu: a 4-by-1 matrix of doubles, the means.
  // sigma: a 4-by-1 matrix of doubles, the standard deviations.
  // exact: a struct with the following fields: expectation, var, S1, S2, S3, S4.
  //
  // Description
  //   This function returns the sensitivity indices of the Sum function.
  //   The second order sensitivity indices Sij, for i,j=1,4 and i different from j are all zeros.
  //   The third order indices Sijk are all zero.
  //   The fourth order indice S1234 is zero.
  //   The total order sensitivity indices are equal to the first order 
  //   indices: STi=Si.
  //
  // This function is defined by 
  //
  // <latex>
  // \begin{eqnarray}
  // Y = a_1 X_1 + a_2 X_2 + a_3 X_3 + a_4 X_4 + a_5
  // \end{eqnarray}
  // </latex>
  //
  // where X1, X2, X3, X4 are normal random variables with means 
  // mu1,mu2,mu3,mu4 and standard variations sigma1,sigma2,sigma3,sigma4.
  //
  // Examples
  // mu=[0 0 0 0]';
  // sigma=[1 2 3 4]';
  // a = [1 1 1 1 0]'
  // exact = nisp_sumsa ( x , a , mu , sigma )
  //
  // mu=[1 2 3 4]';
  // sigma=[5 6 7 8]';
  // a = [1 2 3 4 5]';
  // exact = nisp_sumsa ( x , a , mu , sigma )
  //
  // Authors
  // Copyright (C) 2008-2011 - INRIA - Michael Baudin
  //
  // Bibliography
  // "Sensitivity analysis in practice", Saltelli, Tarantolla, Compolongo, Ratto, Wiley, 2004
  // "An importance quantification technique in uncertainty analysis for computer models", Ishigami, Homma, 1990, Proceedings of the ISUMA'90. First international symposium on uncertainty modelling and Analysis, University of Maryland, USA, pp. 398-403.

    [lhs, rhs] = argn();
    apifun_checkrhs ( "nisp_sumsa" , rhs , 3 : 3 )
    apifun_checklhs ( "nisp_sumsa" , lhs , 0:1 )
    //
    //
    // Check input arguments
    //
    // Check type
    apifun_checktype ( "nisp_sumsa" , a ,   "a" ,  1 , "constant" )
    apifun_checktype ( "nisp_sumsa" , mu ,   "mu" ,  2 , "constant" )
    apifun_checktype ( "nisp_sumsa" , sigma ,   "sigma" ,  3 , "constant" )
    //
    // Check size
    apifun_checkdims ( "nisp_sumsa" , a ,    "a" ,    1 , [5 1] )
    apifun_checkdims ( "nisp_sumsa" , mu ,    "mu" ,    2 , [4 1] )
    apifun_checkdims ( "nisp_sumsa" , sigma ,    "sigma" ,    3 , [4 1] )
    //
    // Check content
    apifun_checkgreq  ( "nisp_sumsa" , sigma ,    "sigma" ,    3 , 0 )
    //

    exact.expectation = sum(mu)+a(5);
    exact.var = sum((a(1:4).*sigma).^2);
    exact.S1 = a(1)^2*sigma(1)^2/exact.var
    exact.S2 = a(2)^2*sigma(2)^2/exact.var
    exact.S3 = a(3)^2*sigma(3)^2/exact.var
    exact.S4 = a(4)^2*sigma(4)^2/exact.var
endfunction

